@@ -19,6 +19,7 @@ from ._models import (
     DEFAULT_CONFIG,
     LOG_EVENT_TYPE,
     PARSER_TYPE,
+    THROTTLE_LOCKER,
     UNSET_VALUE,
     Endpoint,
     GracefulRequest,
@@ -35,6 +36,9 @@ from .exceptions import NonOkResponse, UnexpectedResponse
 
 
 async def _gracefully_throttle(controller: ThrottleController, request_context: GracyRequestContext):
+    if isinstance(request_context.active_config.throttling, Unset):
+        return
+
     if throttling := request_context.active_config.throttling:
         has_been_throttled = True
 
@@ -47,6 +51,9 @@ async def _gracefully_throttle(controller: ThrottleController, request_context:
 
             if wait_per_rule:
                 rule, await_time = max(wait_per_rule, key=lambda x: x[1])
+                if THROTTLE_LOCKER.is_rule_throttled(rule):
+                    await asyncio.sleep(await_time)
+                    continue
 
                 if throttling.log_limit_reached:
                     process_log_throttle(
@@ -57,7 +64,8 @@ async def _gracefully_throttle(controller: ThrottleController, request_context:
                         request_context,
                     )
 
-                await asyncio.sleep(await_time)
+                with THROTTLE_LOCKER.lock_rule(rule):
+                    await asyncio.sleep(await_time)
 
                 if throttling.log_wait_over:
                     process_log_throttle(
@@ -124,17 +132,17 @@ def _check_strictness(active_config: GracyConfig, result: httpx.Response) -> boo
 
 
 def _check_allowed(active_config: GracyConfig, result: httpx.Response) -> bool:
-    if not result.is_success:
-        if active_config.allowed_status_code:
-            if not isinstance(active_config.allowed_status_code, Unset):
-                allowed = active_config.allowed_status_code
-                if not isinstance(allowed, Iterable):
-                    allowed = {allowed}
+    successful = result.is_success
+    if not successful and active_config.allowed_status_code:
+        if not isinstance(active_config.allowed_status_code, Unset):
+            allowed = active_config.allowed_status_code
+            if not isinstance(allowed, Iterable):
+                allowed = {allowed}
 
-                if HTTPStatus(result.status_code) not in allowed:
-                    return False
+            if HTTPStatus(result.status_code) not in allowed:
+                return False
 
-    return True
+    return successful
 
 
 async def _gracify(
